/**
 * Pong.java
 * 1/19/11
 */

import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Timer;

/**
 * Mouse controlled game of Pong.
 * 
 * @author CPSC170A
 */
public class Pong extends JPanel implements ActionListener,
    MouseMotionListener {

    /** Prevent eclipse warning. */
    private static final long serialVersionUID = 1L;

    /** Ball sprite. */
    private Sprite ball;

    /** Paddle sprite. */
    private Sprite paddle;

    /** Game over flag. */
    private boolean gameOver = false;

    /**
     * Runs the application.
     * 
     * @param args command line argument
     * @throws IOException if the an image can not be loaded
     */
    public static void main(final String[] args) throws IOException {
        new Pong();
    }

    /**
     * Constructor, initializes animation window.
     * 
     * @throws IOException if the an image can not be loaded
     */
    public Pong() throws IOException {
        JFrame frame = new JFrame("Pong");
        Timer timer = new Timer(Settings.REFRESH, this);
        Image ballImage = ImageIO.read(new File(Settings.BALL_IMAGE_NAME));
        Image paddleImage = ImageIO.read(new File(Settings.PADDLE_IMAGE_NAME));
        ball = new Sprite(Settings.INIT_BALL_X, Settings.INIT_BALL_Y,
                Settings.BALL_DIAM, Settings.BALL_DIAM,
                Settings.INIT_BALL_DELTA_X, Settings.INIT_BALL_DELTA_Y,
                ballImage);
        paddle = new Sprite(Settings.INIT_PADDLE_X,
                Settings.INIT_PADDLE_Y, Settings.PADDLE_WIDTH,
                Settings.PADDLE_HEIGHT, Settings.INIT_PADDLE_DELTA_X,
                Settings.INIT_PADDLE_DELTA_Y, paddleImage);
        frame.getContentPane().add(this);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        addMouseMotionListener(this);
        setPreferredSize(new Dimension(Settings.PANEL_WIDTH,
                Settings.PANEL_HEIGHT));
        frame.pack();
        timer.start();
        frame.setVisible(true);
    }

    /**
     * Draw the rectangle and the ball.
     * 
     * @param g graphics object to draw to
     */
    public final void paintComponent(final Graphics g) {
        super.paintComponent(g);
        ball.paint(g);
        paddle.paint(g);
        if (gameOver) {
            FontMetrics fontMetrics = g.getFontMetrics();
            int textWidth = fontMetrics.stringWidth(Settings.GAME_OVER_STRING);
            int x = (Settings.PANEL_WIDTH - textWidth) / 2;
            int y = Settings.PANEL_HEIGHT / 2;
            g.setFont(new Font(null, Font.PLAIN, Settings.FONT_SIZE));
            g.drawString(Settings.GAME_OVER_STRING, x, y);
        }
    }

    /**
     * Update the the game every frame of animation.
     * 
     * @param event timer event
     */
    public final void actionPerformed(final ActionEvent event) {
        ball.update();
        if (ball.getX() < 0
                || ball.getX() + ball.getWidth() > Settings.PANEL_WIDTH) {
            ball.setDeltaX(ball.getDeltaX() * -1);
        }
        if (ball.getY() < 0 || ball.intersect(paddle)) {
            ball.setDeltaY(ball.getDeltaY() * -1);
        }
        if (ball.getY() + ball.getHeight() > Settings.PANEL_HEIGHT) {
            ball.setDeltaX(0.0);
            ball.setDeltaY(0.0);
            gameOver = true;
        }
        repaint();
    }

    /**
     * Required by mouse listener, unused.
     * 
     * @param event mouse dragged event
     */
    public void mouseDragged(final MouseEvent event) {
    }

    /**
     * Update the paddle location when the mouse moves.
     * 
     * @param event mouse dragged event
     */
    public final void mouseMoved(final MouseEvent event) {
        paddle.setX(event.getX());
    }

}
