/* 
 * Sprite.java
 * 1/19/11
 */

import java.awt.Graphics;
import java.awt.Image;


/**
 * Class that represents a 2D sprite.
 * 
 * @author CPSC170A
 */
public class Sprite {
    /** Sprite x coordinate. */
    private double x;

    /** Sprite y coordinate. */
    private double y;

    /** Sprite width. */
    private double width;

    /** Sprite height. */
    private double height;

    /** Sprite horizontal velocity. */
    private double deltaX;

    /** Sprite vertical velocity. */
    private double deltaY;

    /** Sprite image when drawn. */
    private Image image;

    /**
     * Initialize a sprite with the specified initial location, size, velocity,
     * and image.
     * 
     * @param initX initial x coordinate of upper left corner
     * @param initY initial y coordinate of upper left corner
     * @param initWidth initial width
     * @param initHeight initial height
     * @param initDeltaX initial x velocity
     * @param initDeltaY initial y velocity
     * @param initImage initial image
     */
    public Sprite(final double initX, final double initY,
            final double initWidth, final double initHeight,
            final double initDeltaX, final double initDeltaY,
            final Image initImage) {
        x = initX;
        y = initY;
        width = initWidth;
        height = initHeight;
        deltaX = initDeltaX;
        deltaY = initDeltaY;
        image = initImage;
    }

    /**
     * Update the location of this sprite based on its velocity.
     */
    public final void update() {
        x += deltaX;
        y += deltaY;
    }

    /**
     * Draw this sprite to the specified graphics object.
     * 
     * @param g graphics object to draw to
     */
    public final void paint(final Graphics g) {
        if (image == null) {
            g.fillRect((int) x, (int) y, (int) width, (int) height);
        } else {
            g.drawImage(image, (int) x, (int) y, (int) width, (int) height,
                    null);
        }
    }

    /**
     * Returns whether this and the specified other sprite intersect.
     * 
     * @param otherSprite the other sprite to test if this sprite is
     * intersecting with
     * @return true if this and otherSprite intersect, false otherwise
     */
    public final boolean intersect(final Sprite otherSprite) {
        boolean intersect = false;
        if (otherSprite.x + otherSprite.width > x
                && otherSprite.x < x + width
                && otherSprite.y + otherSprite.height > y
                && otherSprite.y < y + height) {
            intersect = true;
        }
        return intersect;
    }


    /**
     * Returns the amount, in fractional pixels, that the specified other sprite
     * overlaps with the top edge of this sprite.  If the other sprite does not
     * overlap with the top of this then positive infinity is returned.
     * 
     * @param otherSprite the other sprite to compute overlap with the top of
     * this sprite
     * @return the amount that otherSprite overlaps with the top of this
     */
    public final double topOverlap(final Sprite otherSprite) {
        return Double.POSITIVE_INFINITY;
    }

    /**
     * Returns the amount, in fractional pixels, that the specified other sprite
     * overlaps with the bottom edge of this sprite.  If the other sprite does
     * not overlap with the bottom of this then positive infinity is returned.
     * 
     * @param otherSprite the other sprite to compute overlap with the bottom of
     * this sprite
     * @return the amount that otherSprite overlaps with the bottom of this
     */
    public final double bottompOverlap(final Sprite otherSprite) {
        return Double.POSITIVE_INFINITY;
    }

    /**
     * Returns the amount, in fractional pixels, that the specified other sprite
     * overlaps with the left edge of this sprite.  If the other sprite does not
     * overlap with the left of this then positive infinity is returned.
     * 
     * @param otherSprite the other sprite to compute overlap with the left of
     * this sprite
     * @return the amount that otherSprite overlaps with the left of this
     */
    public final double leftOverlap(final Sprite otherSprite) {
        return Double.POSITIVE_INFINITY;
    }

    /**
     * Returns the amount, in fractional pixels, that the specified other sprite
     * overlaps with the right edge of this sprite.  If the other sprite does
     * not overlap with the right of this then positive infinity is returned.
     * 
     * @param otherSprite the other sprite to compute overlap with the right of
     * this sprite
     * @return the amount that otherSprite overlaps with the right of this
     */
    public final double rightOverlap(final Sprite otherSprite) {
        return Double.POSITIVE_INFINITY;
    }

    /**
     * @return the x
     */
    public final double getX() {
        return x;
    }

    /**
     * @param newX the x to set
     */
    public final void setX(final double newX) {
        x = newX;
    }

    /**
     * @return the y
     */
    public final double getY() {
        return y;
    }

    /**
     * @param newY the y to set
     */
    public final void setY(final double newY) {
        y = newY;
    }

    /**
     * @return the width
     */
    public final double getWidth() {
        return width;
    }

    /**
     * @param newWidth the width to set
     */
    public final void setWidth(final double newWidth) {
        width = newWidth;
    }

    /**
     * @return the height
     */
    public final double getHeight() {
        return height;
    }

    /**
     * @param newHeight the height to set
     */
    public final void setHeight(final double newHeight) {
        height = newHeight;
    }

    /**
     * @return the deltaX
     */
    public final double getDeltaX() {
        return deltaX;
    }

    /**
     * @param newDeltaX the deltaX to set
     */
    public final void setDeltaX(final double newDeltaX) {
        deltaX = newDeltaX;
    }

    /**
     * @return the deltaY
     */
    public final double getDeltaY() {
        return deltaY;
    }

    /**
     * @param newDeltaY the deltaY to set
     */
    public final void setDeltaY(final double newDeltaY) {
        deltaY = newDeltaY;
    }

    /**
     * @return the image
     */
    public final Image getImage() {
        return image;
    }

    /**
     * @param newImage the image to set
     */
    public final void setImage(final Image newImage) {
        image = newImage;
    }

    /**
     * @see java.lang.Object#toString()
     * @return string representing this sprite
     */
    @Override
    public final String toString() {
        return "Sprite [deltaX=" + deltaX + ", deltaY=" + deltaY + ", height="
                + height + ", image=" + image + ", width=" + width + ", x=" + x
                + ", y=" + y + "]";
    }

}
